# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.team import Team
from ..models.serie_id_or_slug import SerieIdOrSlug
from ..models.search_over_dota2_teams import SearchOverDota2Teams
from ..models.range_over_dota2_teams import RangeOverDota2Teams
from ..models.get_dota2_teams_page import GetDota2TeamsPage
from ..models.get_dota2_series_serie_id_or_slug_teams_page import (
    GetDota2SeriesSerieIdOrSlugTeamsPage,
)
from ..models.filter_over_dota2_teams import FilterOverDota2Teams


class Dota2TeamsService(BaseService):

    @cast_models
    def get_dota2_series_serie_id_or_slug_teams(
        self,
        serie_id_or_slug: SerieIdOrSlug,
        filter: FilterOverDota2Teams = None,
        range: RangeOverDota2Teams = None,
        sort: List[any] = None,
        search: SearchOverDota2Teams = None,
        page: GetDota2SeriesSerieIdOrSlugTeamsPage = None,
        per_page: int = None,
    ) -> List[Team]:
        """List teams for the Dota 2 videogame for a given serie

        :param serie_id_or_slug: A serie ID or slug
        :type serie_id_or_slug: SerieIdOrSlug
        :param filter: Options to filter results. String fields are case sensitive <br/>For more information on filtering, see [docs](/docs/filtering-and-sorting#filter)., defaults to None
        :type filter: FilterOverDota2Teams, optional
        :param range: Options to select results within ranges <br/>For more information on ranges, see [docs](/docs/filtering-and-sorting#range)., defaults to None
        :type range: RangeOverDota2Teams, optional
        :param sort: Options to sort results <br/>For more information on sorting, see [docs](/docs/filtering-and-sorting#sort)., defaults to None
        :type sort: List[any], optional
        :param search: Options to search results <br/>For more information on searching, see [docs](/docs/filtering-and-sorting#search)., defaults to None
        :type search: SearchOverDota2Teams, optional
        :param page: Pagination in the form of `page=2` or `page[size]=30&page[number]=2`, defaults to None
        :type page: GetDota2SeriesSerieIdOrSlugTeamsPage, optional
        :param per_page: Equivalent to `page[size]`, defaults to None
        :type per_page: int, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: A list of Dota2 teams
        :rtype: List[Team]
        """

        Validator(SerieIdOrSlug).validate(serie_id_or_slug)
        Validator(FilterOverDota2Teams).is_optional().validate(filter)
        Validator(RangeOverDota2Teams).is_optional().validate(range)
        Validator(any).is_array().is_optional().validate(sort)
        Validator(SearchOverDota2Teams).is_optional().validate(search)
        Validator(GetDota2SeriesSerieIdOrSlugTeamsPage).is_optional().validate(page)
        Validator(int).is_optional().min(1).max(100).validate(per_page)

        serialized_request = (
            Serializer(
                f"{self.base_url}/dota2/series/{{serie_id_or_slug}}/teams",
                self.get_default_headers(),
            )
            .add_path("serie_id_or_slug", serie_id_or_slug)
            .add_query("filter", filter, style="deepObject")
            .add_query("range", range, style="deepObject")
            .add_query("sort", sort)
            .add_query("search", search, style="deepObject")
            .add_query("page", page)
            .add_query("per_page", per_page)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return [Team._unmap(item) for item in response]

    @cast_models
    def get_dota2_teams(
        self,
        filter: FilterOverDota2Teams = None,
        range: RangeOverDota2Teams = None,
        sort: List[any] = None,
        search: SearchOverDota2Teams = None,
        page: GetDota2TeamsPage = None,
        per_page: int = None,
    ) -> List[Team]:
        """List teams for the Dota 2 videogame

        :param filter: Options to filter results. String fields are case sensitive <br/>For more information on filtering, see [docs](/docs/filtering-and-sorting#filter)., defaults to None
        :type filter: FilterOverDota2Teams, optional
        :param range: Options to select results within ranges <br/>For more information on ranges, see [docs](/docs/filtering-and-sorting#range)., defaults to None
        :type range: RangeOverDota2Teams, optional
        :param sort: Options to sort results <br/>For more information on sorting, see [docs](/docs/filtering-and-sorting#sort)., defaults to None
        :type sort: List[any], optional
        :param search: Options to search results <br/>For more information on searching, see [docs](/docs/filtering-and-sorting#search)., defaults to None
        :type search: SearchOverDota2Teams, optional
        :param page: Pagination in the form of `page=2` or `page[size]=30&page[number]=2`, defaults to None
        :type page: GetDota2TeamsPage, optional
        :param per_page: Equivalent to `page[size]`, defaults to None
        :type per_page: int, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: A list of Dota2 teams
        :rtype: List[Team]
        """

        Validator(FilterOverDota2Teams).is_optional().validate(filter)
        Validator(RangeOverDota2Teams).is_optional().validate(range)
        Validator(any).is_array().is_optional().validate(sort)
        Validator(SearchOverDota2Teams).is_optional().validate(search)
        Validator(GetDota2TeamsPage).is_optional().validate(page)
        Validator(int).is_optional().min(1).max(100).validate(per_page)

        serialized_request = (
            Serializer(f"{self.base_url}/dota2/teams", self.get_default_headers())
            .add_query("filter", filter, style="deepObject")
            .add_query("range", range, style="deepObject")
            .add_query("sort", sort)
            .add_query("search", search, style="deepObject")
            .add_query("page", page)
            .add_query("per_page", per_page)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return [Team._unmap(item) for item in response]
