# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.serie import Serie
from ..models.search_over_starcraft2_series import SearchOverStarcraft2Series
from ..models.range_over_starcraft2_series import RangeOverStarcraft2Series
from ..models.get_starcraft_2_series_upcoming_page import (
    GetStarcraft2SeriesUpcomingPage,
)
from ..models.get_starcraft_2_series_running_page import GetStarcraft2SeriesRunningPage
from ..models.get_starcraft_2_series_past_page import GetStarcraft2SeriesPastPage
from ..models.get_starcraft_2_series_page import GetStarcraft2SeriesPage
from ..models.filter_over_starcraft2_series import FilterOverStarcraft2Series


class StarCraft2SeriesService(BaseService):

    @cast_models
    def get_starcraft_2_series(
        self,
        filter: FilterOverStarcraft2Series = None,
        range: RangeOverStarcraft2Series = None,
        sort: List[any] = None,
        search: SearchOverStarcraft2Series = None,
        page: GetStarcraft2SeriesPage = None,
        per_page: int = None,
    ) -> List[Serie]:
        """List series for the  StarCraft 2 videogame

        :param filter: Options to filter results. String fields are case sensitive <br/>For more information on filtering, see [docs](/docs/filtering-and-sorting#filter)., defaults to None
        :type filter: FilterOverStarcraft2Series, optional
        :param range: Options to select results within ranges <br/>For more information on ranges, see [docs](/docs/filtering-and-sorting#range)., defaults to None
        :type range: RangeOverStarcraft2Series, optional
        :param sort: Options to sort results <br/>For more information on sorting, see [docs](/docs/filtering-and-sorting#sort)., defaults to None
        :type sort: List[any], optional
        :param search: Options to search results <br/>For more information on searching, see [docs](/docs/filtering-and-sorting#search)., defaults to None
        :type search: SearchOverStarcraft2Series, optional
        :param page: Pagination in the form of `page=2` or `page[size]=30&page[number]=2`, defaults to None
        :type page: GetStarcraft2SeriesPage, optional
        :param per_page: Equivalent to `page[size]`, defaults to None
        :type per_page: int, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: A list of StarCraft 2 series
        :rtype: List[Serie]
        """

        Validator(FilterOverStarcraft2Series).is_optional().validate(filter)
        Validator(RangeOverStarcraft2Series).is_optional().validate(range)
        Validator(any).is_array().is_optional().validate(sort)
        Validator(SearchOverStarcraft2Series).is_optional().validate(search)
        Validator(GetStarcraft2SeriesPage).is_optional().validate(page)
        Validator(int).is_optional().min(1).max(100).validate(per_page)

        serialized_request = (
            Serializer(
                f"{self.base_url}/starcraft-2/series", self.get_default_headers()
            )
            .add_query("filter", filter, style="deepObject")
            .add_query("range", range, style="deepObject")
            .add_query("sort", sort)
            .add_query("search", search, style="deepObject")
            .add_query("page", page)
            .add_query("per_page", per_page)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return [Serie._unmap(item) for item in response]

    @cast_models
    def get_starcraft_2_series_past(
        self,
        filter: FilterOverStarcraft2Series = None,
        range: RangeOverStarcraft2Series = None,
        sort: List[any] = None,
        search: SearchOverStarcraft2Series = None,
        page: GetStarcraft2SeriesPastPage = None,
        per_page: int = None,
    ) -> List[Serie]:
        """List past StarCraft 2 series

        :param filter: Options to filter results. String fields are case sensitive <br/>For more information on filtering, see [docs](/docs/filtering-and-sorting#filter)., defaults to None
        :type filter: FilterOverStarcraft2Series, optional
        :param range: Options to select results within ranges <br/>For more information on ranges, see [docs](/docs/filtering-and-sorting#range)., defaults to None
        :type range: RangeOverStarcraft2Series, optional
        :param sort: Options to sort results <br/>For more information on sorting, see [docs](/docs/filtering-and-sorting#sort)., defaults to None
        :type sort: List[any], optional
        :param search: Options to search results <br/>For more information on searching, see [docs](/docs/filtering-and-sorting#search)., defaults to None
        :type search: SearchOverStarcraft2Series, optional
        :param page: Pagination in the form of `page=2` or `page[size]=30&page[number]=2`, defaults to None
        :type page: GetStarcraft2SeriesPastPage, optional
        :param per_page: Equivalent to `page[size]`, defaults to None
        :type per_page: int, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: A list of StarCraft 2 series
        :rtype: List[Serie]
        """

        Validator(FilterOverStarcraft2Series).is_optional().validate(filter)
        Validator(RangeOverStarcraft2Series).is_optional().validate(range)
        Validator(any).is_array().is_optional().validate(sort)
        Validator(SearchOverStarcraft2Series).is_optional().validate(search)
        Validator(GetStarcraft2SeriesPastPage).is_optional().validate(page)
        Validator(int).is_optional().min(1).max(100).validate(per_page)

        serialized_request = (
            Serializer(
                f"{self.base_url}/starcraft-2/series/past", self.get_default_headers()
            )
            .add_query("filter", filter, style="deepObject")
            .add_query("range", range, style="deepObject")
            .add_query("sort", sort)
            .add_query("search", search, style="deepObject")
            .add_query("page", page)
            .add_query("per_page", per_page)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return [Serie._unmap(item) for item in response]

    @cast_models
    def get_starcraft_2_series_running(
        self,
        filter: FilterOverStarcraft2Series = None,
        range: RangeOverStarcraft2Series = None,
        sort: List[any] = None,
        search: SearchOverStarcraft2Series = None,
        page: GetStarcraft2SeriesRunningPage = None,
        per_page: int = None,
    ) -> List[Serie]:
        """List running StarCraft 2 series

        :param filter: Options to filter results. String fields are case sensitive <br/>For more information on filtering, see [docs](/docs/filtering-and-sorting#filter)., defaults to None
        :type filter: FilterOverStarcraft2Series, optional
        :param range: Options to select results within ranges <br/>For more information on ranges, see [docs](/docs/filtering-and-sorting#range)., defaults to None
        :type range: RangeOverStarcraft2Series, optional
        :param sort: Options to sort results <br/>For more information on sorting, see [docs](/docs/filtering-and-sorting#sort)., defaults to None
        :type sort: List[any], optional
        :param search: Options to search results <br/>For more information on searching, see [docs](/docs/filtering-and-sorting#search)., defaults to None
        :type search: SearchOverStarcraft2Series, optional
        :param page: Pagination in the form of `page=2` or `page[size]=30&page[number]=2`, defaults to None
        :type page: GetStarcraft2SeriesRunningPage, optional
        :param per_page: Equivalent to `page[size]`, defaults to None
        :type per_page: int, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: A list of StarCraft 2 series
        :rtype: List[Serie]
        """

        Validator(FilterOverStarcraft2Series).is_optional().validate(filter)
        Validator(RangeOverStarcraft2Series).is_optional().validate(range)
        Validator(any).is_array().is_optional().validate(sort)
        Validator(SearchOverStarcraft2Series).is_optional().validate(search)
        Validator(GetStarcraft2SeriesRunningPage).is_optional().validate(page)
        Validator(int).is_optional().min(1).max(100).validate(per_page)

        serialized_request = (
            Serializer(
                f"{self.base_url}/starcraft-2/series/running",
                self.get_default_headers(),
            )
            .add_query("filter", filter, style="deepObject")
            .add_query("range", range, style="deepObject")
            .add_query("sort", sort)
            .add_query("search", search, style="deepObject")
            .add_query("page", page)
            .add_query("per_page", per_page)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return [Serie._unmap(item) for item in response]

    @cast_models
    def get_starcraft_2_series_upcoming(
        self,
        filter: FilterOverStarcraft2Series = None,
        range: RangeOverStarcraft2Series = None,
        sort: List[any] = None,
        search: SearchOverStarcraft2Series = None,
        page: GetStarcraft2SeriesUpcomingPage = None,
        per_page: int = None,
    ) -> List[Serie]:
        """List upcoming StarCraft 2 series

        :param filter: Options to filter results. String fields are case sensitive <br/>For more information on filtering, see [docs](/docs/filtering-and-sorting#filter)., defaults to None
        :type filter: FilterOverStarcraft2Series, optional
        :param range: Options to select results within ranges <br/>For more information on ranges, see [docs](/docs/filtering-and-sorting#range)., defaults to None
        :type range: RangeOverStarcraft2Series, optional
        :param sort: Options to sort results <br/>For more information on sorting, see [docs](/docs/filtering-and-sorting#sort)., defaults to None
        :type sort: List[any], optional
        :param search: Options to search results <br/>For more information on searching, see [docs](/docs/filtering-and-sorting#search)., defaults to None
        :type search: SearchOverStarcraft2Series, optional
        :param page: Pagination in the form of `page=2` or `page[size]=30&page[number]=2`, defaults to None
        :type page: GetStarcraft2SeriesUpcomingPage, optional
        :param per_page: Equivalent to `page[size]`, defaults to None
        :type per_page: int, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: A list of StarCraft 2 series
        :rtype: List[Serie]
        """

        Validator(FilterOverStarcraft2Series).is_optional().validate(filter)
        Validator(RangeOverStarcraft2Series).is_optional().validate(range)
        Validator(any).is_array().is_optional().validate(sort)
        Validator(SearchOverStarcraft2Series).is_optional().validate(search)
        Validator(GetStarcraft2SeriesUpcomingPage).is_optional().validate(page)
        Validator(int).is_optional().min(1).max(100).validate(per_page)

        serialized_request = (
            Serializer(
                f"{self.base_url}/starcraft-2/series/upcoming",
                self.get_default_headers(),
            )
            .add_query("filter", filter, style="deepObject")
            .add_query("range", range, style="deepObject")
            .add_query("sort", sort)
            .add_query("search", search, style="deepObject")
            .add_query("page", page)
            .add_query("per_page", per_page)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return [Serie._unmap(item) for item in response]
